# coding: utf-8

import datetime

from django.db import models
from django.utils import simplejson as json
from django.db.models.signals import pre_delete
from django.dispatch import receiver

from djcelery.models import PeriodicTask, CrontabSchedule
from m3.actions import ApplicationLogicException
from m3.db import BaseEnumerate

from kinder.core.audit_log.managers import AuditLog
from kinder.core.models import BaseReplicatedModel
from kinder.core.unit.models import UnitKind
from kinder.core.unit.models import Unit

from utils import ViewList
from constants import (
    AGE_CATEGORIES_FULL, AGE_CATEGORIES_FULL_CURRENT,
    APPLICATION_STATUSES, AGE_CATEGORIES_FULL_LIST,
    BENEFIT_TYPES, HEALTH_NEEDS, DOO_TYPES, DELIVERY_TYPES,
    GROUP_TYPES, AGE_CATEGORIES_CUT, NO, AGE_CATEGORIES_FULL_MAP,
    AGE_CATEGORIES_CUT_LIST)
import settings as gs_settings


@receiver(pre_delete, sender=Unit)
def delete_unit(instance, **kwargs):

    GisdoUnit.objects.filter(unit=instance).delete()


class InstallationType(BaseEnumerate):

    REGIONAL = 1
    MUNICIPAL = 2

    values = {
        REGIONAL: u'Региональная',
        MUNICIPAL: u'Муниципальная'
    }


class GisdoUnit(BaseReplicatedModel):
    """
    Модель, для расширения учреждений
    """
    unit = models.OneToOneField(Unit, parent_link=True, related_name='gisdo')
    not_on_federal_report = models.BooleanField(
        verbose_name=u"Не отправлять фед.отчетность",
        default=False
    )
    doo_identity = models.CharField(
        max_length=20, verbose_name=u'Идентификатор ДОО для шины', null=True)

    audit_log = AuditLog(['modified'])

    # идентификатор ДОО
    def create_doo_identity(self):
        """композитный идентификатор ДОО, заполняется следующим образом:
        xxyyyyyyyyyyyyzzzz,
        xx - код региона (2 знака) из настроек проекта
        yyyyyyyyyyyy - цифры кода ОКТМО (11 знаков)
        zzzz - номер ДОО (поле doo_number)
        """
        octmo_max_length = 11
        doo_number_max_length = 4
        region_max_length = 2

        if self.unit.kind.id != UnitKind.DOU:
            return
        if not gs_settings.REGION_CODE:
            raise ApplicationLogicException(u'Не задан код региона')
        if len(str(gs_settings.REGION_CODE)) > region_max_length:
            raise ApplicationLogicException(
                u'Длина кода региона превышает максимальное значение')

        identity = str(gs_settings.REGION_CODE).rjust(region_max_length, '0')

        unit_mo = self.unit.get_mo()
        octmo = unit_mo.octmo
        if octmo:
            identity += octmo.rjust(octmo_max_length, '0')
        else:
            raise ApplicationLogicException(
                u'В МО %s не задан код ОКТМО' % unit_mo.display())

        identity += str(self.unit.doo_number).rjust(doo_number_max_length, '0')

        # Так сделано потому, что теперь не изменяем идентификатор,
        # если он начинается с тринадцати нулей.
        if not identity.startswith('0'*(region_max_length + octmo_max_length)):
            self.doo_identity = identity


class ReportForm(BaseReplicatedModel):
    """
    Форма отчетности "Федеральная отчетность"
    """
    date = models.DateTimeField(
        verbose_name=u'Дата создания', auto_now_add=True)
    sent_time = models.DateTimeField(
        verbose_name=u'Время отправки', null=True)
    user = models.ForeignKey(
        'users.UserProfile', verbose_name=u'Пользователь', null=True)
    unit = models.ForeignKey(
        'unit.Unit', verbose_name=u'Учреждение', null=True)
    xml = models.TextField(
        verbose_name=u'Сформированная XML', null=True)
    in_progress = models.BooleanField(
        default=False, verbose_name=u'Сбор отчета')

    class Meta:
        verbose_name = u"Форма 'Федеральная отчетность'"
        verbose_name_plural = u"Формы 'Федеральная отчетность'"

    def __unicode__(self):
        return u"%s от %s" % (self._meta.verbose_name, self.date)

    @property
    def presentation(self):

        date = self.date.strftime("%d.%m.%Y")

        return u"{date}, {unit}".format(date=date, unit=self.unit)

    @property
    def progress(self):
        return u'В процессе сборки' if self.in_progress else u'Собран'

    @property
    def sent(self):
        return u'Отправлен' if self.sent_time else u'Не отправлен'


class ReportFormRow(BaseReplicatedModel):
    """
    Строка отчета для конкретного учреждения
    """
    report = models.ForeignKey(ReportForm, verbose_name=u'Форма отчетности')
    unit = models.ForeignKey('unit.Unit', verbose_name=u'Учреждение')

    applications = models.TextField(
        verbose_name=u'Информация по заявлениям', null=True)
    queues = models.TextField(
        verbose_name=u'Информация по очередям', null=True)
    enrolled = models.TextField(
        verbose_name=u'Информация о зачисленных детях', null=True)
    capacities = models.TextField(
        verbose_name=u'Информация о свободных местах', null=True)

    class Meta:
        verbose_name = u"Показатель по учреждению"
        verbose_name_plural = u"Показатели по учреждению"

    def __unicode__(self):
        return u"%s %s на %s" % (
            self._meta.verbose_name,
            self.unit.display(),
            self.report.date if self.report else '-')


class ScheduleSettings(models.Model):
    """
    Модель настроек celery beat
    """
    PK = 1
    push_login = models.CharField(
        u'Логин для отправки', max_length=30, null=True)
    push_password = models.CharField(
        u'Пароль для отправки', max_length=30, null=True)

    time = models.TimeField(verbose_name=u'Время начала', null=True)
    is_active = models.BooleanField(
        verbose_name=u'Включение автоматического сбора',
        default=False)
    resend_count = models.PositiveSmallIntegerField(
        verbose_name=u'Количество повторных отправок при неудаче',
        default=3)
    resend_after_time = models.PositiveSmallIntegerField(
        verbose_name=u'Запускать автоматическую отправку '
                     u'после неудачной попытки через',
        default=0)  # число минут
    zip_xml = models.BooleanField(
        verbose_name=u'Кодировать данные при отправке в ZIP',
        default=False)
    async_send = models.BooleanField(
        verbose_name=u'Асинхронная отправка отчета', default=False)
    system_name = models.CharField(
        u'Наименование системы', max_length=250, null=True)
    email = models.EmailField(
        u'E-mail контактного лица', max_length=30, null=True)
    installation_type = models.SmallIntegerField(
        choices=InstallationType.get_choices(), null=True)

    def save(self, force_insert=False, force_update=False, using=None):
        self.pk = self.PK
        super(ScheduleSettings, self).save(force_update, force_update, using)

    @staticmethod
    def get_settings():
        ss, created = ScheduleSettings.objects.get_or_create(
            id=ScheduleSettings.PK, defaults={'id': ScheduleSettings.PK})
        return ss

    @classmethod
    def get_interval_and_count_retries(cls):
        settings = cls.get_settings()

        interval = settings.resend_after_time
        now = datetime.datetime.now()
        tomorrow = (datetime.datetime(now.year, now.month, now.day, 0, 0, 0) +
                    datetime.timedelta(days=1))
        # вычисляем сколько раз имеет смысл совершить
        # повторных попыток до конца сегодняшнего дня
        if interval:
            max_count_retries_before_tomorrow = (tomorrow - now).seconds / (60 * interval)
        else:
            # если interval = 0 (значение "Нет" на форме), то сохраняем
            # старую реализацию, то есть не переотправляем
            max_count_retries_before_tomorrow = 0

        return interval, max_count_retries_before_tomorrow


class TaskScheduler(models.Model):
    """
    Планировщик для сбора данных
    """
    periodic_task = models.ForeignKey(PeriodicTask)

    @staticmethod
    def schedule_every(task_name, hour, minute, args='[]', kwargs='{}'):
        """
        schedules a task by name
        """
        ptask_name = "%s_%s" % (task_name, datetime.datetime.now())
        crontab_schedules = CrontabSchedule.objects.filter(hour=hour, minute=minute,
                                                           day_of_week='*', day_of_month='*', month_of_year='*')
        crontab_schedule = (crontab_schedules[0]
                            if crontab_schedules.exists()
                            else CrontabSchedule.objects.create(hour=hour, minute=minute))
        ptask = PeriodicTask.objects.create(name=ptask_name,
                                            task=task_name,
                                            crontab=crontab_schedule,
                                            args=args,
                                            kwargs=kwargs,
                                            queue=gs_settings.CELERY_WORKER_TASK_QUEUE)
        return TaskScheduler.objects.create(periodic_task=ptask)

    def stop(self):
        self.periodic_task.enabled = False
        self.periodic_task.save()

    def start(self):
        self.periodic_task.enabled = True
        self.periodic_task.save()

    def terminate(self):
        self.stop()
        self.delete()
        self.periodic_task.delete()